-- CPLD Interface for SelectMAP Configuration (VIRTEX) from Parallel EPROM.
-- Designer: Carl Carmichael
-- Company: XILINX, Inc.
-- Date: Jan 20, 1999.
-- module: prom_map.
-- components: OBUFT.
-- Implementation: XC9536-VQ44

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity prom_map is
port (	CCLK, DONE, PROG, INIT, BUSY : in STD_LOGIC;
	CS, WRITE : out STD_LOGIC;
	ADDR : out STD_LOGIC_VECTOR(19 downto 0));
end prom_map;

ARCHITECTURE interface OF prom_map IS

component OBUFT
port ( T, I : in STD_LOGIC;
	O : out STD_LOGIC);
end component;

SIGNAL control, write_strobe, chip_select : STD_LOGIC;
SIGNAL address	: STD_LOGIC_VECTOR(19 downto 0);

BEGIN

write_control: process(CCLK, PROG)
BEGIN
   IF (PROG = '0') THEN
	control <= '0';
   ELSIF (CCLK'event and CCLK='1') THEN
	control <= INIT;
   END IF;
END process;

address_counter: process(CCLK, PROG)
BEGIN
   IF (PROG = '0') THEN
        address <= "00000000000000000000";
   ELSIF (CCLK'event and CCLK='1') THEN
      IF (INIT = '1' and control = '1' and BUSY = '0') THEN
	address <= address + "00000000000000000001";
      ELSE
	address <= address;
      END IF;
   END IF;
END process;

chip_select <= NOT(control);
write_strobe <= NOT(control);

u0: OBUFT port map (O=>ADDR(0), T=>DONE, I=>address(0));
u1: OBUFT port map (O=>ADDR(1), T=>DONE, I=>address(1));
u2: OBUFT port map (O=>ADDR(2), T=>DONE, I=>address(2));
u3: OBUFT port map (O=>ADDR(3), T=>DONE, I=>address(3));
u4: OBUFT port map (O=>ADDR(4), T=>DONE, I=>address(4));
u5: OBUFT port map (O=>ADDR(5), T=>DONE, I=>address(5));
u6: OBUFT port map (O=>ADDR(6), T=>DONE, I=>address(6));
u7: OBUFT port map (O=>ADDR(7), T=>DONE, I=>address(7));
u8: OBUFT port map (O=>ADDR(8), T=>DONE, I=>address(8));
u9: OBUFT port map (O=>ADDR(9), T=>DONE, I=>address(9));
u10: OBUFT port map (O=>ADDR(10), T=>DONE, I=>address(10));
u11: OBUFT port map (O=>ADDR(11), T=>DONE, I=>address(11));
u12: OBUFT port map (O=>ADDR(12), T=>DONE, I=>address(12));
u13: OBUFT port map (O=>ADDR(13), T=>DONE, I=>address(13));
u14: OBUFT port map (O=>ADDR(14), T=>DONE, I=>address(14));
u15: OBUFT port map (O=>ADDR(15), T=>DONE, I=>address(15));
u16: OBUFT port map (O=>ADDR(16), T=>DONE, I=>address(16));
u17: OBUFT port map (O=>ADDR(17), T=>DONE, I=>address(17));
u18: OBUFT port map (O=>ADDR(18), T=>DONE, I=>address(18));
u19: OBUFT port map (O=>ADDR(19), T=>DONE, I=>address(19));
u20: OBUFT port map (O=>CS, T=>DONE, I=>chip_select);
u21: OBUFT port map (O=>WRITE, T=>DONE, I=>write_strobe);

END;
